import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'package:rook_sdk_core/rook_sdk_core.dart';
import 'package:rook_sdk_health_connect/src/domain/enums/hc_availability_status.dart';
import 'package:rook_sdk_health_connect/src/domain/enums/health_permission.dart';
import 'package:rook_sdk_health_connect/src/platform/rook_sdk_health_connect_method_channel.dart';

abstract class RookSdkHealthConnectPlatform extends PlatformInterface {
  RookSdkHealthConnectPlatform() : super(token: _token);

  static final Object _token = Object();

  static RookSdkHealthConnectPlatform _instance =
      MethodChannelRookSdkHealthConnect();

  static RookSdkHealthConnectPlatform get instance => _instance;

  static set instance(RookSdkHealthConnectPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  Future<void> enableNativeLogs();

  Future<void> setConfiguration(RookConfiguration rookConfiguration);

  Future<void> initRook();

  Future<void> updateUserID(String userID);

  Future<void> clearUserID();

  Future<void> syncUserTimeZone();

  Future<HCAvailabilityStatus> checkAvailability();

  Future<void> openHealthConnectSettings();

  Future<bool> checkPermissions(HealthPermission permission);

  Future<void> requestPermissions(HealthPermission permission);

  Future<bool> shouldSyncSleepSummariesFor(DateTime date);

  Future<void> syncSleepSummary(DateTime date);

  Future<bool> shouldSyncPhysicalSummariesFor(DateTime date);

  Future<void> syncPhysicalSummary(DateTime date);

  Future<bool> shouldSyncBodySummariesFor(DateTime date);

  Future<void> syncBodySummary(DateTime date);

  Future<void> syncPendingSummaries();

  Future<void> syncPhysicalEvents(DateTime date);

  Future<void> syncBloodGlucoseEvents(DateTime date);

  Future<void> syncBloodPressureEvents(DateTime date);

  Future<void> syncBodyMetricsEvents(DateTime date);

  Future<void> syncBodyHeartRateEvents(DateTime date);

  Future<void> syncPhysicalHeartRateEvents(DateTime date);

  Future<void> syncHydrationEvents(DateTime date);

  Future<void> syncNutritionEvents(DateTime date);

  Future<void> syncBodyOxygenationEvents(DateTime date);

  Future<void> syncPhysicalOxygenationEvents(DateTime date);

  Future<void> syncTemperatureEvents(DateTime date);

  Future<void> syncPendingEvents();

  Future<bool> isStepsTrackerAvailable();

  Future<bool> isStepsTrackerActive();

  Future<bool> hasStepsTrackerPermissions();

  Future<void> requestStepsTrackerPermissions();

  Future<void> startStepsTracker();

  Future<void> stopStepsTracker();

  Future<int> getTodaySteps();
}
